<?php

/**
 * The Controller for CaseTrackerCase entities
 */
class CaseTrackerCaseController extends EntityAPIController {

  public function __construct() {
    parent::__construct('casetracker_case');
  }

  /**
   *
   * @param $type
   *   The machine-readable type of the case.
   *
   * @return
   *   A case object with all default fields initialized.
   */
  public function create(array $values = array()) {
    // Add values that are specific to our Case
    $values += array(
      'pid' => '',
      'is_new' => TRUE,
      'title' => '',
      'language' => '',
      'uid' => '',
      'created' => '',
      'changed' => '',
    );

    $case = parent::create($values);
    return $case;
  }

  /**
   *
   * Implements EntityAPIControllerInterface.
   *
   * Generate an array for rendering the given entities.
   *
   * @param $entities
   *   An array of entities to render.
   * @param $view_mode
   *   View mode, e.g. 'full', 'teaser'...
   * @param $langcode
   *   (optional) A language code to use for rendering. Defaults to the global
   *   content language of the current request.
   * @param $page
   *   (optional) If set will control if the entity is rendered: if TRUE
   *   the entity will be rendered without its title, so that it can be embeded
   *   in another context. If FALSE the entity will be displayed with its title
   *   in a mode suitable for lists.
   *   If unset, the page mode will be enabled if the current path is the URI
   *   of the entity, as returned by entity_uri().
   *   This parameter is only supported for entities which controller is a
   *   EntityAPIControllerInterface.
   * @return
   *   The renderable array, keyed by entity name or numeric id.
   */
  public function view($entities, $view_mode = 'full', $langcode = NULL, $page = NULL) {

    // For Field API and entity_prepare_view, the entities have to be keyed by
    // (numeric) id.
    $entities = entity_key_array_by_property($entities, $this->idKey);
    if (!empty($this->entityInfo['fieldable'])) {
      field_attach_prepare_view($this->entityType, $entities, $view_mode);
    }
    entity_prepare_view($this->entityType, $entities);
    $langcode = isset($langcode) ? $langcode : $GLOBALS['language_content']->language;

    $view = array();
    foreach ($entities as $entity) {

      $build = $this->themeEntity($entity);
      $build[] = entity_build_content($this->entityType, $entity, $view_mode, $langcode);

      //Allow modules to modify the structured entity.
      drupal_alter(array($this->entityType . '_view', 'entity_view'), $build, $this->entityType);
      $key = isset($entity->{$this->idKey}) ? $entity->{$this->idKey} : NULL;
      $view[$this->entityType][$key] = $build;
    }
    return $view;
  }

  /**
   * Define how an entity will be themed and return the build array with it.
   * @param  CaseTrackerCase $entity
   * @return array
   */
  protected function themeEntity($entity) {
    $build = array();
    $build['view_mode_full'] = array(
      '#theme' => 'casetracker_case_full_view_mode',
      '#CaseTrackerCase' => $entity,
    );
    return $build;
  }

  /**
   * Sums the amount of Cases grouped by priority and optionally filter by project.
   * @param  int $project_id
   * @return array of CaseTrackerCase
   */
  public function getTotalAmountByPriority($project_id = NULL) {

    $query = "SELECT p.field_casetracker_case_priority_value priority, count(c.cid) total
              FROM {casetracker_case} c
              JOIN {field_data_field_casetracker_case_priority} p ON p.entity_type = 'casetracker_case' AND p.entity_id = c.cid
              ";

    if ($project_id != NULL) {
      $query .= " JOIN {field_data_field_casetracker_project_ref} proj ON proj.entity_type = 'casetracker_case' AND proj.entity_id = c.cid AND proj.field_casetracker_project_ref_target_id = :pid";
    }

    $query .= " GROUP BY priority";

    if ($project_id != NULL) {
      $results = db_query($query, array(':pid' => $project_id))->fetchAll();
    }
    else {
      $results = db_query($query)->fetchAll();
    }

    return $results;
  }

  /**
   * Sums the amount of Cases grouped by bundle and optionally filter by project.
   * @param  int $project_id
   * @return array of CaseTrackerCase
   */
  public function getTotalAmountByBundle($project_id = NULL) {

    $query = "SELECT t.label bundle, count(c.cid) total
              FROM {casetracker_case} c
              INNER JOIN {casetracker_case_type} t ON t.type = c.type";

    if ($project_id != NULL) {
      $query .= " JOIN {field_data_field_casetracker_project_ref} proj ON proj.entity_type = 'casetracker_case' AND proj.entity_id = c.cid AND proj.field_casetracker_project_ref_target_id = :pid";
    }

    $query .= " GROUP BY c.type";

    if ($project_id != NULL) {
      $results = db_query($query, array(':pid' => $project_id))->fetchAll();
    }
    else {
      $results = db_query($query)->fetchAll();
    }

    return $results;
  }

}
